#!/usr/bin/perl -w
# Copyright (C) 2008 Miroslav Jezbera <jezz@hkfree.org>
# This file is part of LQueues - simple bandwidth manager for linux.
#
# LQueues is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# LQueues is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with LQueues.  If not, see <http://www.gnu.org/licenses/agpl.html>.

use strict;

my $configFile = 'queues.conf';
my $tc = '/sbin/tc';
my $defaultSpeed = '174080';
my $maxLinkSpeed = '174080';
my $shaperType = 'HFSC'; # HTB, HFSC

my %devices = ();

sub systemWithLog
	{
	my $cmd = $_[0];
	if (system($cmd) != 0)
		{
		printf STDERR "system($cmd) has failed!\n";
		}
	}

sub setupSystem
	{
	&systemWithLog("for mod in sch_htb sch_hfsc sch_sfq cls_u32; do modprobe \$mod; done");
	}

sub setupDeviceHTB
	{
	my $device = $_[0];
	if (not exists $devices{$device})
		{
		system("$tc qdisc del dev $device root");
		&systemWithLog("$tc qdisc add dev $device root handle 1: htb r2q 10 default 0");
		$devices{$device} = 1;
		}
	}

sub setupDeviceHFSC
	{
	my $device = $_[0];
	if (not exists $devices{$device})
		{
		system("$tc qdisc del dev $device root");
		&systemWithLog("$tc qdisc add dev $device root handle 1: hfsc default fffe");
		&systemWithLog("$tc class add dev $device parent 1: classid 1:ffff hfsc sc m2 ${maxLinkSpeed}kbit ul m2 ${maxLinkSpeed}kbit");
		&systemWithLog("$tc class add dev $device parent 1:ffff classid 1:fffe hfsc sc m2 ${defaultSpeed}kbit ul m2 ${maxLinkSpeed}kbit");
		&systemWithLog("$tc qdisc add dev $device parent 1:fffe sfq perturb 10");
		$devices{$device} = 1;
		}
	}

sub setupClassHTB
	{
	my ($id, $direction, $iface, $speed, $ipsRec) = @_;
	my @ips = split /\s+/, $ipsRec;

	if ($iface !~ /^$/ and @ips)
		{
		&setupDeviceHTB($iface);
		$speed = $defaultSpeed if ($speed eq '' or $speed == 0);
		$speed *= 1024;
		&systemWithLog("$tc class add dev $iface parent 1: classid 1:$id htb rate ${speed}bit burst 32kbit");
		&systemWithLog("$tc qdisc add dev $iface parent 1:$id handle ${id}0: sfq perturb 10 limit 64");
		foreach my $ip (@ips)
			{
			&systemWithLog("$tc filter add dev $iface parent 1:0 prio 10 protocol ip u32 match ip $direction $ip flowid 1:$id");
			}
		}
	}

sub setupClassHFSC
	{
	my ($id, $direction, $iface, $speed, $ipsRec) = @_;
	my @ips = split /\s+/, $ipsRec;

	if ($iface !~ /^$/ and @ips)
		{
		&setupDeviceHFSC($iface);
		$speed = $defaultSpeed if ($speed eq '' or $speed == 0);
		$speed *= 1024;
		&systemWithLog("$tc class add dev $iface parent 1:ffff classid 1:$id hfsc sc m2 ${speed}bit ul m2 ${speed}bit");
		&systemWithLog("$tc qdisc add dev $iface parent 1:$id handle ${id}0: sfq perturb 10 limit 64");
		foreach my $ip (@ips)
			{
			&systemWithLog("$tc filter add dev $iface parent 1:0 prio 10 protocol ip u32 match ip $direction $ip flowid 1:$id");
			}
		}
	}

open CONFIG, "$configFile" or die "Can't open config file, because: $!\n";

# shaper type setting
my $setupClass = \&setupClassHTB;
if ($shaperType eq 'HFSC')
	{
	$setupClass = \&setupClassHFSC;
	}

&setupSystem();
my $lineNum = 1;
while (<CONFIG>)
	{
	chomp;
	next if /^\s*([#%].*)?$/;
	my ($name, $inIface, $outIface, $downSpeed, $upSpeed, $ips) = split /\s*,\s*/, $_;
	&$setupClass($lineNum, "dst", $inIface, $downSpeed, $ips);
	&$setupClass($lineNum, "src", $outIface, $upSpeed, $ips);
	$lineNum++;
	}

close CONFIG;
